from app.sample_models import SpreadsheetModel, SpreadsheetResponse
from fastapi import APIRouter, UploadFile, File, HTTPException
import logging
from app.services.spreadsheet_service import SampleSpreadsheetImporter, SpreadsheetImportError
from fastapi.responses import FileResponse
import os

router = APIRouter()
logger = logging.getLogger(__name__)

@router.get("/download-template", response_class=FileResponse)
async def download_template():
    # No changes here; just serves a static file
    current_dir = os.path.dirname(__file__)
    template_path = os.path.join(current_dir, "../../downloads/V7_TELLSamplesSpreadsheetTemplate.xlsx")
    if not os.path.exists(template_path):
        raise HTTPException(status_code=404, detail="Template file not found.")
    return FileResponse(template_path, filename="template.xlsx",
                        media_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")

@router.post("/upload", response_model=SpreadsheetResponse)
async def upload_file(file: UploadFile = File(...)):
    try:
        logger.info(f"Received file: {file.filename}")

        # Validate file type
        if not file.filename.endswith('.xlsx'):
            logger.error("Invalid file format")
            raise HTTPException(status_code=400, detail="Invalid file format. Please upload an .xlsx file.")

        # Process spreadsheet
        importer = SampleSpreadsheetImporter()
        validated_model, errors, raw_data = importer.import_spreadsheet_with_errors(file)

        # Collect dewar, puck, and sample names
        dewars = {sample.dewarname for sample in validated_model if sample.dewarname}
        pucks = {sample.puckname for sample in validated_model if sample.puckname}
        samples = {sample.crystalname for sample in validated_model if sample.crystalname}

        # Construct response data
        response_data = SpreadsheetResponse(
            data=validated_model,
            errors=errors,
            raw_data=raw_data,
            dewars_count=len(dewars),
            dewars=list(dewars),
            pucks_count=len(pucks),
            pucks=list(pucks),
            samples_count=len(samples),
            samples=list(samples)
        )

        logger.info(f"Returning response: {response_data.dict()}")
        return response_data
    except SpreadsheetImportError as e:
        logger.error(f"Spreadsheet import error: {str(e)}")
        raise HTTPException(status_code=400, detail=str(e))
    except Exception as e:
        logger.error(f"Failed to process file: {str(e)}")
        raise HTTPException(status_code=500, detail=f"Failed to upload file. Please try again. {str(e)}")
